; Border Trix
; Copyright (c) Jan 4, 2012 Andrew Owen

; many thanks to everyone in the #zx channel on coldfront.net for testing

; requires a stack in non-contended RAM

org 49102
	jr 		switch_on		; jump to the setup code

org 49104
switch_off:
	di						; restore IM1 mode
	im		1
	ei
	ret

switch_on:
	di						; disable interrupts

	push	af				; save the registers
	push	bc
	push	de
	push	hl

	ld		hl, 0xfff4		; this is the address called by the JR at 0xffff
	ld		(hl), 0xc3		; JP
	inc		hl
	ld		(hl), 0x00		; low byte of im_main address
	inc		hl
	ld		(hl), 0xc0		; high byte of im_main address
	ld		l, 0xff
	ld		(hl), 0x18		; JR

	ld		a, 0x3b			; high byte of vector table
	ld		i, a
	
	ld		hl, screen		; copy the screen in
	ld		de, 16384
	ld		bc, 6912
	ldir
	
	pop		hl				; restore registers
	pop		de
	pop		bc
	pop		af

enable_im2:	
	im		2				; switch it on
	ei						; enable interrupts
	ret						; return to BASIC

org	49152
im_main:
	push	af				; save registers
	push	bc
	push	de
	push	hl

	ld		hl, 337			; wait for the first visible line
	push	af
	pop		af
	nop
	nop

wait:						; inline version of the delay loop
	dec		hl
	ld		a, h
	or		l
	jr		nz, wait

; set up colours

	ld		a,  0x07		; A = white
	ld		bc, 0x02fe		; B = red (C = port #FE)
	ld		de, 0x0604		; D = yellow, E = green
	ld		hl, 0x0500		; H = cyan, L = black

; upper text

	call	row0h			; hilight
	call	row0
	call	row0
	call	row0

	call	row1
	call	row1
	call	row1
	call	row1

	call	row2h			; hilight
	call	row2
	call	row2
	call	row2

	call	row1 
	call	row1
	call	row1
	call	row1

	call	row3h			; hilight
	call	row3x
	call	row3y
	call	row3

; three rainbow bars

	nop			; 28 t-state pause
	nop
	nop
	nop
	nop
	nop
	nop

	ld		hl, 39
	call	rainbow

	ld		hl, 14
	call	rainbow

	ld		hl, 14
	call	rainbow
	
	ld		hl, 32
	call	delay
	
	ld		hl, 0x0500		; restore colour values
	ld		a, 0x07

	ld		r, a			; 9 t-state pause

; lower text

	call	row4h			; hilight
	call	row4
	call	row4
	call	row4

	call	row5
	call	row5
	call	row5
	call	row5

	call	row6h			; hilight
	call	row6
	call	row6
	call	row6

	call	row7 
	call	row7
	call	row7
	call	row7

	call	row8h			; hilight
	call	row4
	call	row4

row8a:						; special last row (minimize the t-state count)

	ld		(65534), a		; 17 t-state pause (unused address)
	nop						;

	nop						; nop here for issue 3 or later ULAs
	out		(c), h
	out		(c), h
	out		(c), h
	out		(c), l
	out		(c), e
	out		(c), e
	out		(c), e
;	nop						; nop here for issue 2 or earlier ULAs
	out		(c), l
	out		(c), d
	out		(c), l
	out		(c), b
	out		(c), b
	out		(c), b
	out		(c), l

	pop		hl				; restore registers
	pop		de
	pop		bc
	pop		af
	
	jp		0x0038			; jump into ROM interrupt handler

; each row of the border text is assembled from the following subroutines
; effectively the code gives you a 13x20 block map. Each block is 24x1 pixels.
; you can use any of 6 colours in the map (A, B, D, E, H, L)

; upper border

row0:
	nop						; nop here for issue 3 or later ULAs
	out		(c), b
	out		(c), b
	out		(c), l
	out		(c), d
	out		(c), l
	out		(c), d
	out		(c), l
;	nop						; nop here for issue 2 or earlier ULAs
	out		(c), e
	out		(c), e
	out		(c), l
	out		(c), h
	out		(c), l
	out		(c), h
	out		(c), l
	nop
	ld		a, 7			; 21 t-state pause (must hold colour value)
	ld		a, 7
	ld		a, 7
	ret

row0h:						; highlight
	nop						; nop here for issue 3 or later ULAs
	out		(c), a
	out		(c), a
	out		(c), l
	out		(c), a
	out		(c), l
	out		(c), a
	out		(c), l
;	nop						; nop here for issue 2 or earlier ULAs
	out		(c), a
	out		(c), a
	out		(c), l
	out		(c), a
	out		(c), l
	out		(c), a
	out		(c), l
	nop
	ld		a, 7			; 21 t-state pause (must hold colour value)
	ld		a, 7
	ld		a, 7
	ret

row1:
	nop						; nop here for issue 3 or later ULAs
	out		(c), b
	out		(c), l
	out		(c), l
	out		(c), d
	out		(c), l
	out		(c), d
	out		(c), l
;	nop						; nop here for issue 2 or earlier ULAs
	out		(c), e
	out		(c), l
	out		(c), l
	out		(c), h
	out		(c), l
	out		(c), h
	out		(c), l
	nop
	ld		a, 7			; 21 t-state pause (must hold colour value)
	ld		a, 7
	ld		a, 7
	ret

row2:
	nop						; nop here for issue 3 or later ULAs
	out		(c), b
	out		(c), l
	out		(c), l
	out		(c), d
	out		(c), d
	out		(c), d
	out		(c), l
;	nop						; nop here for issue 2 or earlier ULAs
	out		(c), e
	out		(c), e
	out		(c), l
	out		(c), h
	out		(c), l
	out		(c), h
	out		(c), l
	nop
	ld		a, 7			; 21 t-state pause (must hold colour value)
	ld		a, 7
	ld		a, 7
	ret

row2h:
	nop						; nop here for issue 3 or later ULAs
	out		(c), b
	out		(c), l
	out		(c), l
	out		(c), d
	out		(c), a
	out		(c), d
	out		(c), l
;	nop						; nop here for issue 2 or earlier ULAs
	out		(c), e
	out		(c), a
	out		(c), l
	out		(c), h
	out		(c), l
	out		(c), h
	out		(c), l
	nop
	ld		a, 7			; 21 t-state pause (must hold colour value)
	ld		a, 7
	ld		a, 7
	ret

row3:
	nop						; nop here for issue 3 or later ULAs
	out		(c), b
	out		(c), b
	out		(c), l
	out		(c), d
	out		(c), l
	out		(c), d
	out		(c), l
;	nop						; nop here for issue 2 or earlier ULAs
	out		(c), e
	out		(c), e
	out		(c), l
	out		(c), h
	out		(c), h
	out		(c), l
	out		(c), l
	nop
	ld		a, 7			; 21 t-state pause (must hold colour value)
	ld		a, 7
	ld		a, 7
	ret

row3h:
	nop						; nop here for issue 3 or later ULAs
	out		(c), b
	out		(c), a
	out		(c), l
	out		(c), d
	out		(c), l
	out		(c), d
	out		(c), l
;	nop						; nop here for issue 2 or earlier ULAs
	out		(c), e
	out		(c), a
	out		(c), l
	out		(c), h
	out		(c), a
	out		(c), h
	out		(c), l
	nop
	ld		a, 7			; 21 t-state pause (must hold colour value)
	ld		a, 7
	ld		a, 7
	ret

row3x:
	nop						; nop here for issue 3 or later ULAs
	out		(c), b
	out		(c), b
	out		(c), l
	out		(c), d
	out		(c), l
	out		(c), d
	out		(c), l
;	nop						; nop here for issue 2 or earlier ULAs
	out		(c), e
	out		(c), e
	out		(c), l
	out		(c), h
	out		(c), h
	nop
	nop
	out		(c), l
	out		(c), l
	ld		a, 7			; 17 t-state pause (must hold colour values)
	ld		hl, 0x0500
	ret

row3y:
	nop						; nop here for issue 3 or later ULAs
	out		(c), b
	out		(c), b
	out		(c), l
	out		(c), d
	out		(c), l
	out		(c), d
	out		(c), l
;	nop						; nop here for issue 2 or earlier ULAs
	out		(c), e
	out		(c), e
	out		(c), l
	out		(c), h
	out		(c), h
	nop
	out		(c), l
	out		(c), l
	ld		a, 7			; 21 t-state pause (must hold colour value)
	ld		a, 7
	ld		a, 7
	ret

; lower border

row4:
	nop						; nop here for issue 3 or later ULAs
	out		(c), h
	out		(c), h
	out		(c), h
	out		(c), l
	out		(c), e
	out		(c), e
	out		(c), e
;	nop						; nop here for issue 2 or earlier ULAs
	out		(c), l
	out		(c), d
	out		(c), l
	out		(c), b
	out		(c), b
	out		(c), b
	out		(c), l
	nop
	ld		a, 7			; 21 t-state pause (must hold colour value)
	ld		a, 7
	ld		a, 7
	ret

row4h:						; highlight
	nop						; nop here for issue 3 or later ULAs
	out		(c), a
	out		(c), a
	out		(c), a
	out		(c), l
	out		(c), a
	out		(c), a
	out		(c), a
;	nop						; nop here for issue 2 or earlier ULAs
	out		(c), l
	out		(c), a
	out		(c), l
	out		(c), a
	out		(c), a
	out		(c), a
	out		(c), l
	nop
	ld		a, 7			; 21 t-state pause (must hold colour value)
	ld		a, 7
	ld		a, 7
	ret

row5:
	nop						; nop here for issue 3 or later ULAs
	out		(c), l
	out		(c), l
	out		(c), h
	out		(c), l
	out		(c), e
	out		(c), l
	out		(c), e
;	nop						; nop here for issue 2 or earlier ULAs
	out		(c), l
	out		(c), d
	out		(c), l
	out		(c), l
	out		(c), l
	out		(c), b
	out		(c), l
	nop
	ld		a, 7			; 21 t-state pause (must hold colour value)
	ld		a, 7
	ld		a, 7
	ret

row6:
	nop						; nop here for issue 3 or later ULAs
	out		(c), h
	out		(c), h
	out		(c), h
	out		(c), l
	out		(c), e
	out		(c), l
	out		(c), e
;	nop						; nop here for issue 2 or earlier ULAs
	out		(c), l
	out		(c), d
	out		(c), l
	out		(c), b
	out		(c), b
	out		(c), b
	out		(c), l
	nop
	ld		a, 7			; 21 t-state pause (must hold colour value)
	ld		a, 7
	ld		a, 7
	ret

row6h:
	nop						; nop here for issue 3 or later ULAs
	out		(c), a
	out		(c), a
	out		(c), h
	out		(c), l
	out		(c), e
	out		(c), l
	out		(c), e
;	nop						; nop here for issue 2 or earlier ULAs
	out		(c), l
	out		(c), d
	out		(c), l
	out		(c), a
	out		(c), a
	out		(c), b
	out		(c), l
	nop
	ld		a, 7			; 21 t-state pause (must hold colour value)
	ld		a, 7
	ld		a, 7
	ret

row7:
	nop						; nop here for issue 3 or later ULAs
	out		(c), h
	out		(c), l
	out		(c), l
	out		(c), l
	out		(c), e
	out		(c), l
	out		(c), e
;	nop						; nop here for issue 2 or earlier ULAs
	out		(c), l
	out		(c), d
	out		(c), l
	out		(c), b
	out		(c), l
	out		(c), l
	out		(c), l
	nop
	ld		a, 7			; 21 t-state pause (must hold colour value)
	ld		a, 7
	ld		a, 7
	ret

row8h:
	nop						; nop here for issue 3 or later ULAs
	out		(c), h
	out		(c), a
	out		(c), a
	out		(c), l
	out		(c), e
	out		(c), a
	out		(c), e
;	nop						; nop here for issue 2 or earlier ULAs
	out		(c), l
	out		(c), d
	out		(c), l
	out		(c), b
	out		(c), a
	out		(c), a
	out		(c), l
	nop
	ld		a, 7			; 21 t-state pause (must hold colour value)
	ld		a, 7
	ld		a, 7
	ret

rainbow:					; rainbow border (called three times) 
	call	delay

	ld		a, 7
	out		(c), a
	ld		hl, 7
	call	delay

	ld		a, 0
	out		(c), a
	ld		hl, 15
	call	delay

	ld		a, 7
	out		(c), a
	ld		hl, 16
	call	delay

	ld		a, 0
	out		(c), a
	ld		hl, 6
	call	delay

	ld		a, 7
	out		(c), a
	ld		hl, 15
	call	delay

	ld		a, 5
	out		(c), a
	ld		hl, 7
	call	delay

	ld		a, 7
	out		(c), a
	ld		hl, 15
	call	delay

	ld		a, 5
	out		(c), a
	ld		hl, 15
	nop
	nop
	call	delay

	ld		a, 7
	out		(c), a
	ld		hl, 7
	call	delay

	ld		a, 5
	out		(c), a
	ld		hl, 15
	nop
	call	delay

;;;

	ld		a, 4
	out		(c), a
	ld		hl, 7
	call	delay

	ld		a, 5
	out		(c), a
	ld		hl, 14
	call	delay

	ld		a, 4
	out		(c), a
	ld		hl, 16
	call	delay

	ld		a, 5
	out		(c), a
	ld		hl, 6
	call	delay

	ld		a, 4
	out		(c), a
	ld		hl, 16
	call	delay

	ld		a, 6
	out		(c), a
	ld		hl, 7
	call	delay

	ld		a, 4
	out		(c), a
	ld		hl, 15
	call	delay

	ld		a, 6
	out		(c), a
	ld		hl, 15
	nop
	nop
	call	delay

	ld		a, 4
	out		(c), a
	ld		hl, 7
	call	delay

	ld		a, 6
	out		(c), a
	ld		hl, 15
	nop
	nop
	nop
	call	delay

;;;

	ld		a, 2
	out		(c), a
	ld		hl, 7
	call	delay

	ld		a, 6
	out		(c), a
	ld		hl, 14
	call	delay

	ld		a, 2
	out		(c), a
	ld		hl, 16
	call	delay

	ld		a, 6
	out		(c), a
	ld		hl, 6
	call	delay

	ld		a, 2
	out		(c), a
	ld		hl, 15
	call	delay

	ld		a, 3
	out		(c), a
	ld		hl, 7
	call	delay

	ld		a, 2
	out		(c), a
	ld		hl, 15
	call	delay

	ld		a, 3
	out		(c), a
	ld		hl, 15
	nop
	nop
	call	delay

	ld		a, 2
	out		(c), a
	ld		hl, 7
	call	delay

	ld		a, 3
	out		(c), a
	ld		hl, 16
	call	delay

;;;

	ld		a, 1
	out		(c), a
	ld		hl, 7
	call	delay

	ld		a, 3
	out		(c), a
	ld		hl, 14
	call	delay

	ld		a, 1
	out		(c), a
	ld		hl, 16
	call	delay

	ld		a, 3
	out		(c), a
	ld		hl, 6
	call	delay

	ld		a, 1
	out		(c), a
	ld		hl, 15
	call	delay

	ld		a, 0
	out		(c), a
	ld		hl, 7
	call	delay

	ld		a, 1
	out		(c), a
	ld		hl, 15
	call	delay

	ld		a, 0
	out		(c), a
	ld		hl, 15
	nop
	nop
	call	delay

	ld		a, 1
	out		(c), a
	ld		hl, 7
	call	delay

	xor		a
	out		(c), a
	nop
	ret

delay:						; delay subroutine
	dec		hl
	ld		a, h
	or		l
	jr		nz, delay
	ret

screen:
	incbin	"twist.scr"

; the following code is created by the setup code

;org 65524
;vector:
;	jp		im_main			; jump to main interrupt handler

;org 65535
;	defb	24				; combined with first ROM byte forms JR vector
